
% *********************************************************************
%                        ORTools - Example 1
% *********************************************************************
% Reconstructing the prior austenite microstructure from lath martensite
% Find the detailed MTEX version on the details of the reconstruction here:
% https://mtex-toolbox.github.io/MaParentGrainReconstruction.html
% *********************************************************************
% Dr. Azdiar Gazder, 2020, azdiaratuowdotedudotau
% Dr. Frank Niessen, 2020, contactatfniessendotcom
% (Remove "dot" and "at" to make this email address valid)
% *********************************************************************
clc; close all; clear all;
currentFolder;
screenPrint('StartUp','ORTools - Example 1: DP800');
%% Initialize MTEX
% Startup and set some settings
startup_mtex;
% Default directories - Do not modify
Ini.dataPath = [pwd,'\data\'];
Ini.cifPath = [Ini.dataPath,'input\cif\'];
Ini.ebsdPath = [Ini.dataPath,'input\ebsd\'];
Ini.texturePath = [Ini.dataPath,'output\texture\'];
Ini.imagePath = [Ini.dataPath,'output\images\'];
%% Specify Crystal, Specimen Symmetries and file names
% crystal symmetry
CS = {... 
  'notIndexed',...
  crystalSymmetry('432', [2.9 2.9 2.9], 'mineral', 'Iron - Alpha', 'color', [0.53 0.81 0.98]),...
  crystalSymmetry('432', [3.6 3.6 3.6], 'mineral', 'Iron - Gamma', 'color', [0 0 1])};
% plotting convention
setMTEXpref('xAxisDirection','east');
setMTEXpref('zAxisDirection','intoPlane');
setMTEXpref('FontSize',14);
% Specify File Names
% path to files
pname = 'file_path';
% which files to be imported
fname = [pname '\file_name'];
% Import the Data
% create an EBSD variable containing the data
ebsd = EBSD.load(fname,CS,'interface','ang',...
  'convertEuler2SpatialReferenceFrame','setting 2');

%% create variable ebsd containing data of Iron - Alpha
ebsd1=ebsd;
ebsd=ebsd('Iron - Alpha');
figure()
plot(ebsd,log(ebsd.prop.iq),'figSize','large')
mtexColorMap Gray
figure()
plot(ebsd,ebsd.orientations,'figSize','large','faceAlpha',0.5)

%% Identifying Daughter Grains
%screenPrint('SegmentStart','Computing and filtering grains');
[grains,ebsd.grainId] = calcGrains(ebsd,'angle',2*degree);
%Remove small grains and re-identify
ebsd(grains(grains.grainSize < 5)) = [];
[grains,ebsd.grainId] = calcGrains(ebsd,'angle',2*degree);
%Visualize grains
figure()
plot(ebsd,ebsd.orientations,'figSize','large')
hold on
plot(grains.boundary,'linewidth',2)
hold off
%% Ignore ferritic regions for reconstruction (based on GAIQ)
x=numel(grains.id);
for i=1:x
    if mean(ebsd(grains(i)).prop.iq)>68000
       ebsd(grains(i)).phase=0;                
    end
end 
% Reidentify grains with ferrite switched off:
[grains,ebsd.grainId] = calcGrains(ebsd,'angle',2*degree);
% Smooth grain boundaries
grains = smooth(grains,20); % For making smooth boundaries
% Visualize
figure()
plot(ebsd,ebsd.orientations,'figSize','large')
hold on
plot(grains.boundary)
hold off

%% Set up the job
job = setParentGrainReconstructor(ebsd,grains,Ini.cifPath);
% initial guess for the parent to child orientation relationship
job.p2c = orientation.KurdjumovSachs(job.csParent, job.csChild);
%job.p2c = orientation.NishiyamaWassermann(job.csParent, job.csChild)
%job.p2c = orientation.GreningerTroiano(job.csParent, job.csChild)
% optimize OR
job.calcParent2Child
% misfit histogram
% close all
figure()
histogram(job.fit./degree)
xlabel('disorientation angle')
% Let's now plot some information about the OR
ORinfo(job.p2c);

%% compute the misfit for all child to child grain neighbours
[fit,c2cPairs] = job.calcGBFit;
% select grain boundary segments by grain ids
[gB,pairId] = job.grains.boundary.selectByGrainId(c2cPairs);
% plot the child phase
figure()
plot(ebsd,ebsd.orientations,'figSize','large','faceAlpha',0.5)
% and on top of it the boundaries colorized by the misfit and
% scale fit between 0 and 1 - required for edgeAlpha
hold on;
plot(gB, 'edgeAlpha', (fit(pairId) ./ degree - 2.5)./2 ,'linewidth',2);
hold off
% Set up a similarity graph
job.calcGraph('threshold',2.5*degree,'tolerance',2.5*degree);
figure()
plot(ebsd,ebsd.orientations,'figSize','large','faceAlpha',0.5)
hold on;
job.plotGraph('linewidth',2)
hold off

%% Cluster graph into components
job.clusterGraph('inflationPower',1.6)
job.calcParentFromGraph
figure()
plot(job.parentGrains,job.parentGrains.meanOrientation)

%% Fill the holes with voting algorithm
% We can now use the already confidently reconstructed gamma grains to 
% vote for the gamma orientation of not yet reconstructed alpha grains
% We iterate this 5 times ...
for k = 1:5 % do this three times
  % compute votes
  job.calcGBVotes('noC2C');
  % compute parent orientations from votes
  job.calcParentFromVote('minFit',7.5*degree)
end
% plot the result
figure()
plot(job.parentGrains,job.parentGrains.meanOrientation)
%% Merge grains and inclusions
% merge grains with similar orientation
job.mergeSimilar('threshold',7.5*degree);
% plot the result
figure()
plot(job.parentGrains,job.parentGrains.meanOrientation)
% merge inclusion
job.mergeInclusions('maxSize',50);
figure()
plot(job.parentGrains,job.parentGrains.meanOrientation)

%% Remove very small parent grains
parentGrains = smooth(job.parentGrains,10);
parentGrains(parentGrains.grainSize<15) = [];
figure()
plot(ebsd,log(ebsd.prop.iq),'figSize','large')
mtexColorMap Gray
hold on
plot(parentGrains.boundary,'linewidth',2)
hold off

%% Variant analysis
% Now that we have both the alpha and the associated prior gamma
% orientations, we can conduct variant analysis.
% Let's plot the possible variants in a pole figure
%plotPDF_variants(job);
% We then calculate the variant IDs of all alpha grains ...
job.calcVariants;
% ... and plot them
plotMap_variants(job,'linewidth',3);
% The same can be done for the packets
plotMap_packets(job,'linewidth',3);


%% outline a specific parent grain
parentGrains = smooth(job.parentGrains,10);
grainSelected = parentGrains(parentGrains.findByLocation([2.3,3.1]));
hold on
figure()
%plot(parentGrains(125).boundary)
plot(grainSelected.boundary,'linewidth',3,'lineColor','black')
hold off


%% Correction and switching ON ferrite

% Slightly modified the script. The spaces on the scan edges were getting included in grains, while they are not
%included in PAG reconstruction, resulting in size mismatch error while using %job.calcParentEBSD. 

y=numel(job.ebsd.id);
 for i=1:y
     z=max(find(job.grains.phaseId==1)); %#ok<MXFND>
     if job.ebsd((i)).grainId>numel(job.grains.phaseId)
        job.ebsd((i)).grainId=z;                
     end
     if ebsd(i).phase==0  % Switch ON ferrite
        ebsd(i).phase=1;
        job.ebsd(i).phase=1;
     end  
 end
 
%% Parent EBSD Reconstruction

parentEBSD = job.calcParentEBSD; % Run this command standalone, some issues
figure()
plot(parentEBSD('Iron - Gamma'),parentEBSD('Iron - Gamma').orientations,'figSize','large')
% Denoise the Parent Map
[parentGrains, parentEBSD.grainId] = calcGrains(parentEBSD('indexed'),'angle',3*degree);
% remove very small grains
parentEBSD(parentGrains(parentGrains.grainSize<10)) = [];
% redo grain reconstrucion
[parentGrains, parentEBSD.grainId] = calcGrains(parentEBSD('indexed'),'angle',3*degree);
parentGrains = smooth(parentGrains,15);
figure()
plot(ebsd('indexed'),ebsd('indexed').orientations,'figSize','large')
hold on
plot(parentGrains.boundary,'lineWidth',2)
hold off
% fill the holes
F = halfQuadraticFilter;
parentEBSD = smooth(parentEBSD('indexed'),F,'fill',parentGrains);
% plot the parent map
figure()
plot(parentEBSD('Iron - Gamma'),parentEBSD('Iron - Gamma').orientations,'figSize','large')
% with grain boundaries
hold on
plot(parentGrains.boundary,'lineWidth',2)
hold off
%% Reconstruct parent EBSD 
% We can finally obtain the reconstructed EBSD data
parentEBSD = job.calcParentEBSD;
% And plot it with the prior beta grain boundaries
figure;
plot(parentEBSD(job.csParent),parentEBSD(job.csParent).orientations);
hold on; 
plot(job.grains.boundary,'lineWidth',3)

%% Save images
saveImage(Ini.imagePath);

%% Check the gamma grains interactively by clicking on them
grainClick(job,parentEBSD);

% Some nice pictures: PAG boundaries on top of IQ map
figure()
plot(ebsd,log(ebsd.prop.iq),'figSize','large')
mtexColorMap Gray
hold on
plot(job.parentGrains.boundary,'linewidth',2)
hold off

% Highlighting a particulat PAG on top of IQ map
grainSelected = job.parentGrains(job.parentGrains.findByLocation([2.3,3.1]));
figure()
plot(ebsd,log(ebsd.prop.iq),'figSize','large')
mtexColorMap Gray
hold on
plot(grainSelected.boundary,'linewidth',3,'lineColor','red') 

%% Change to square grid 
% define a square unit cell
unitCell = [-.0125 -.0125; -.0125 .0125; .0125 .0125; .0125 -.0125]; %can use even smaller unitcell

% use the square unit cell for gridify
ebsdS1 = parentEBSD.gridify('unitCell',unitCell);
ebsdS2 = ebsd.gridify('unitCell',unitCell);             %make sure ebsd and parentEBSD have equal number of IDs

% visualize the result
figure()
plot(ebsdS1,log(ebsdS1.prop.iq),'figSize','large')
mtexColorMap Gray

%% Export
%Files placed in the OR-Tools folder
export_ang(ebsdS1,'new_fila_path_1')    
export_ang(ebsdS2,'new_file_path_2')